@@ -1,7 +1,7 @@
 /*
  * Value.java
  *
- * Copyright (c) 2006-2007 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2006-2008 Operational Dynamics Consulting Pty Ltd
  * 
  * The code in this file, and the library it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -11,8 +11,9 @@
  */
 package org.gnome.glib;
 
+import org.freedesktop.bindings.Constant;
 import org.freedesktop.bindings.Debug;
-import org.freedesktop.bindings.Proxy;
+import org.freedesktop.bindings.Pointer;
 import org.gnome.gdk.Pixbuf;
 
 /**
@@ -43,13 +44,13 @@ import org.gnome.gdk.Pixbuf;
  * <p>
  * <i>Since instances of Java classes are their own identity, we do not need
  * to directly represent <code>GType</code> and <code>GValue</code> as
- * separate classes. We implement <code>GType</code> as a characteristic
- * that any</i> <code>Value</code> <i>or</i> <code>Object</code> <i>has.</i>
+ * separate classes. We implement <code>GType</code> as a characteristic that
+ * any</i> <code>Value</code> <i>or</i> <code>Object</code> <i>has.</i>
  * 
  * @author Andrew Cowie
  * @since 4.0.0
  */
-public class Value extends Proxy
+public class Value extends Pointer
 {
     /*
      * The second argument is a hack to create a different overload of
@@ -63,19 +64,6 @@ public class Value extends Proxy
         }
     }
 
-    /**
-     * Call the release() method to free the GValue, then carry on to
-     * {@link org.freedesktop.bindings.Proxy#finalize() Proxy's finalize()}.
-     */
-    /*
-     * This is a placeholder to remind us of the cleanup actions that will be
-     * necessary, irrespective of the finalizer technique used.
-     */
-    protected void finalize() {
-        release();
-        super.finalize();
-    }
-
     /**
      * By design, we are the owner of all the GValues because we allocate the
      * memory for them in (for example) the native implementation of
@@ -92,8 +80,8 @@ public class Value extends Proxy
     /**
      * Create an empty Value without initializing it's type. For use in
      * methods like TreeModel's
-     * {@link org.gnome.gtk.TreeModel#getValue(org.gnome.gtk.TreeIter, org.gnome.gtk.DataColumnString) getValue()}
-     * family, which use a blank Value internally.
+     * {@link org.gnome.gtk.TreeModel#getValue(org.gnome.gtk.TreeIter, org.gnome.gtk.DataColumnString)
+     * getValue()} family, which use a blank Value internally.
      * 
      * Not public API!
      */
@@ -141,6 +129,10 @@ public class Value extends Proxy
         this(GValue.createValue(value), true);
     }
 
+    protected Value(double value) {
+        this(GValue.createValue(value), true);
+    }
+
     protected Value(Pixbuf pixbuf) {
         this(GValue.createValue(pixbuf), true);
     }
@@ -155,7 +147,7 @@ public class Value extends Proxy
     protected Object getObject() {
         return GValue.getObject(this);
     }
-    
+
     protected Pixbuf getPixbuf() {
         return GValue.getPixbuf(this);
     }
@@ -164,11 +156,23 @@ public class Value extends Proxy
         return GValue.getFloat(this);
     }
 
+    protected double getDouble() {
+        return GValue.getDouble(this);
+    }
+
     protected Value(long value) {
         this(GValue.createValue(value), true);
     }
 
+    protected Value(Constant value) {
+        this(GValue.createValue(value), true);
+    }
+
     protected long getLong() {
         return GValue.getLong(this);
     }
+
+    protected Constant getEnum() {
+        return GValue.getEnum(this);
+    }
 }
