@@ -1,9 +1,9 @@
 /*
  * Thing.java
  *
- * Copyright (c) 2007 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2007-2008 Operational Dynamics Consulting Pty Ltd
  * 
- * The code in this file, and the library it is a part of, are made available
+ * The code in this file, and the program it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
  * version 2" See the LICENCE file for the terms governing usage and
  * redistribution.
@@ -97,10 +97,10 @@ public abstract class Thing
         return blacklisted;
     }
 
-    private static HashMap things;
+    private static HashMap<String, Thing> things;
 
     static {
-        things = new HashMap(400);
+        things = new HashMap<String, Thing>(500);
 
         register(new FundamentalThing("none", "void", "void", "void"));
         register(new FundamentalThing("gchar", "char", "char", "jchar"));
@@ -124,6 +124,7 @@ public abstract class Thing
         register(new FundamentalThing("gint32", "int", "int", "jint"));
         register(new FundamentalThing("guint32", "int", "int", "jint"));
         register(new FundamentalThing("gssize", "int", "int", "jint"));
+        register(new FundamentalThing("gsize", "long", "long", "jlong"));
         register(new FundamentalThing("gint64", "long", "long", "jlong"));
         register(new FundamentalThing("glong", "long", "long", "jlong"));
         register(new FundamentalThing("gulong", "long", "long", "jlong"));
@@ -141,6 +142,7 @@ public abstract class Thing
          * and for string arrays
          */
         register(new StringArrayThing("gchar**"));
+        register(new StringArrayThing("char**"));
 
         /* these seem a bit harder */
         register(new FixmeThing("const-gchar*[]"));
@@ -157,6 +159,7 @@ public abstract class Thing
         register(new FundamentalArrayThing("gfloat*", "gfloat"));
         register(new FundamentalArrayThing("gdouble*", "gdouble"));
         register(new FundamentalArrayThing("gboolean*", "gboolean"));
+        register(new FundamentalArrayThing("gsize*", "gsize"));
         // and so on
 
         /*
@@ -226,21 +229,8 @@ public abstract class Thing
          */
         register(new BlacklistedThing("guchar**"));
 
-        /*
-         * GList/GSList and related typedefs... FIXME change this when we
-         * properly map the list and array returns.
-         */
-        register(new FixmeThing("GList*"));
-        register(new FixmeThing("GSList*"));
         register(new FixmeThing("AtkAttributeSet*"));
 
-        /*
-         * FUTURE no Cairo bindings, yet
-         */
-        register(new BlacklistedThing("cairo_t*"));
-        register(new BlacklistedThing("cairo_surface_t*"));
-        register(new BlacklistedThing("cairo_font_options_t*"));
-
         /*
          * And what on earth are...
          */
@@ -287,7 +277,7 @@ public abstract class Thing
          * Lookup the type. This will work most of the time...
          */
 
-        stored = (Thing) things.get(gType);
+        stored = things.get(gType);
 
         if (stored != null) {
             return stored;
@@ -301,7 +291,7 @@ public abstract class Thing
 
         if (gType.startsWith("const-")) {
             bareGType = gType.substring(6);
-            stored = (Thing) things.get(bareGType);
+            stored = things.get(bareGType);
 
             if (stored != null) {
                 dupe = stored.createConstVariant();
@@ -311,7 +301,7 @@ public abstract class Thing
             }
         } else if (gType.startsWith("GList-") || gType.startsWith("GSList-")) {
             bareGType = gType.split("-")[1];
-            stored = (Thing) things.get(bareGType);
+            stored = things.get(bareGType);
 
             if (stored != null) {
                 dupe = stored.createListVariant(gType);
@@ -321,11 +311,12 @@ public abstract class Thing
             }
         } else if (gType.endsWith("*")) {
             bareGType = gType.substring(0, gType.length() - 1);
-            stored = (Thing) things.get(bareGType);
+            stored = things.get(bareGType);
 
             /*
              * we don't support arrays of arrays yet. the !(stored instanceof
-             * ArrayThing) is needed to prevent multiple recursion on *
+             * ArrayThing) is needed to prevent multiple recursion on the
+             * pointer star.
              */
             if ((stored != null) && !(stored instanceof ArrayThing)) {
                 dupe = stored.createArrayVariant();
@@ -335,11 +326,6 @@ public abstract class Thing
             }
         }
 
-        /*
-         * TODO if we finally difference between * and [] we need to add more
-         * code here
-         */
-
         /*
          * If we're still stuck, then that is would be fatal, except that we
          * create a dummy type as a placeholder so we can proceed with the
@@ -477,6 +463,20 @@ public abstract class Thing
         return name;
     }
 
+    /**
+     * 
+     * @param name
+     * @param callerOwnsReturn
+     *            Whether the caller owns the return value. It is either 'f'
+     *            (caller doesn't own return), 't' (caller own return) or 'l'
+     *            (return type is a list/array... and only the list itself is
+     *            own, not its contents)
+     * @return
+     */
+    String jniReturnCleanup(String name, char callerOwnsReturn) {
+        return null;
+    }
+
     /**
      * Little utility function so that when aborting out of a C function
      * (because an Exception has been thrown) the correct "emtpy" value is
@@ -546,7 +546,7 @@ public abstract class Thing
         Thing t;
 
         try {
-            t = (Thing) this.getClass().newInstance();
+            t = this.getClass().newInstance();
         } catch (InstantiationException ie) {
             throw new RuntimeException("No nullary constructor available in " + this.getClass() + "?",
                     ie);
