@@ -10,6 +10,10 @@
  */
 package org.gnome.gtk;
 
+import java.io.FileNotFoundException;
+
+import org.gnome.gdk.Pixbuf;
+
 /**
  * Test characteristic getters and setters to ensure correct values are
  * retrieved.
@@ -51,6 +55,11 @@ public class ValidateProperties extends TestCaseGtk
         assertEquals(0.1f, v.getFloat(), 0.0001);
     }
 
+    public final void testDoubleValues() {
+        final Value v = new Value(42.2d);
+        assertEquals(42.2d, v.getDouble(), 0.0001);
+    }
+
     /**
      * Test changing the label of a Button and getting the label back.
      */
@@ -221,4 +230,108 @@ public class ValidateProperties extends TestCaseGtk
         assertTrue(width > 1);
         assertTrue(height > 1);
     }
+
+    /*
+     * FUTURE Given the elaborate relationship between resize(),
+     * setDefaultSize(), and setSizeRequest(), this could probably be a lot
+     * more stringent.
+     */
+    public final void testResizeRestrictions() {
+        final Window w;
+
+        w = new Window();
+
+        try {
+            w.resize(0, 1);
+            fail("Shouldn't accept sub-minimum sizes.");
+        } catch (IllegalArgumentException iae) {
+            // good
+        }
+
+        try {
+            w.resize(1, 0);
+            fail("Shouldn't accept sub-minimum sizes.");
+        } catch (IllegalArgumentException iae) {
+            // good
+        }
+
+        w.resize(1, 1);
+    }
+
+    /*
+     * TODO make this a bit more meaningful; at the moment all it is doing is
+     * making sure that no FatalErrors are blown by switching from an Image
+     * storing one type of image to another; (yes, one would expect this to
+     * Just Work; but that's the point of this test as it stands so far).
+     */
+    public final void testImageResetting() throws FileNotFoundException {
+        final Image i;
+        final Pixbuf data;
+
+        i = new Image(Stock.REFRESH, IconSize.LARGE_TOOLBAR);
+
+        data = new Pixbuf("src/bindings/java-gnome_Icon.png");
+        i.setImage(data);
+        i.clear();
+    }
+
+    public final void testEntryStyleProperties() {
+        final Entry entry;
+
+        entry = new Entry();
+
+        /*
+         * Check default
+         */
+        assertEquals(true, GtkEntry.getHasFrame(entry));
+
+        /*
+         * Check our setter
+         */
+
+        entry.setHasFrame(false);
+        assertEquals(false, GtkEntry.getHasFrame(entry));
+    }
+
+    public final void testButtonFocus() {
+        final Button button;
+
+        button = new Button("Hello");
+
+        /*
+         * Check default
+         */
+        assertEquals(true, GtkButton.getFocusOnClick(button));
+
+        /*
+         * Check our setter
+         */
+
+        button.setFocusOnClick(false);
+        assertEquals(false, GtkButton.getFocusOnClick(button));
+    }
+
+    public final void testDefaultProperty() {
+        final Button button;
+        final Window window;
+
+        button = new Button("Hello");
+
+        /*
+         * Check can-default's default :)
+         */
+
+        assertEquals(false, button.getCanDefault());
+
+        button.setCanDefault(true);
+        assertEquals(true, button.getCanDefault());
+
+        /*
+         * And make sure that this doesn't fail
+         */
+
+        window = new Window();
+        window.add(button);
+        button.grabDefault();
+    }
 }
